import { setXRow } from "@/utils/tzc";
/**
* @function 单位名称转换
* @description 后台数据中的单位名称转换成汇总表中对应的单位名称
* @param name {string} 单位名称
* @param isGB {bool} 是否干部
* @return string
* @author cwl 2024年5月22日 15:22:03
*/
function getUnitName(name, isGB = false) {
    var newName;
    //洗煤厂特殊处理
    switch (name) {
        case "后勤服务一科":
            newName = "（1）后勤服务一科";
            break;
        case "后勤服务二科":
            newName = "（2）后勤服务二科";
            break;
        case "精神病院劳务":
            newName = "（3）精神病院劳务";
            break;
        case "低热值煤炭":
            newName = "（4）低热值煤炭";
            break;
        case "后勤办公室":
            newName = "（5）后勤办公室";
            break;
        case "保卫科":
            newName = "（1）保卫科（矿）";
            break;
        case "直属警察队":
            newName = "（2）直属警察队（社区）";
            break;
        case "物资管理科":
            newName="回收厂";
            break;
        case "煤类产品管理检测科":
            newName="煤管科";
            break;
        default:
            newName = name;
            break;
    }
    if (name == "洗煤厂" && isGB) {
        newName = "6.制造费洗煤";
    } else if (name == "洗煤厂" && !isGB) {
        newName = "4.基本生产洗煤";
    } else if (isGB) {
        newName += "--干部";
    }

    return newName;
}
/**
* @function 唐山矿工资数据处理
* @description 
* @param jsonData 导入的数据
* @return 处理后的luckysheet的一维数组
* @author cwl 2024年5月22日 15:22:03
*/
export function machGzdata(list,jsonData) {
    var CompositeTable = jsonData;
    //后台返回数据
    CompositeTable[0].data = luckysheet.transToData(CompositeTable[0].celldata);
    var dList = setXRow(CompositeTable[0].data);//汇总表中单位集合包含行数
    list.forEach((item) => {
        let dName = item.dep;
        let uNameGB = getUnitName(dName, true);
        let uNameGR = getUnitName(dName);

        //如果含有干部和工人数
        if (item.worker=="0" && item.cadre=="0") {
            var r = dList[uNameGR];
            item.r = r;
            item.v = item.value;
        } else {
            //干部人数
            var newItem = {};
            newItem.dep = uNameGB;
            newItem.r = dList[uNameGB];
            newItem.v = item.cadre;
            newItem.c = item.c;
            list.push(newItem);

            //工人数
            item.dep = uNameGR;
            item.r = dList[uNameGR];
            item.v = item.worker;
            
        }
    });
    // 对比 c 和 r 是否与 CompositeTable[0]["celldata"] 中相应对象相同
    for (let k = 0; k < CompositeTable[0]["celldata"].length; k++) {
        for (let i = 0; i < list.length; i++) {
            if (
                list[i].r == CompositeTable[0]["celldata"][k].r &&
                list[i].c == CompositeTable[0]["celldata"][k].c
            ) {
                // 如果相同，则进行替换
                CompositeTable[0]["celldata"][k].v.v = list[i].v;
            }
        }
    }
    return CompositeTable;
}
