package com.ruoyi.system.service.impl;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.ActSupplies;
import com.ruoyi.system.domain.ActSuppliesDetails;
import com.ruoyi.system.domain.ActSuppliesMaterialCode;
import com.ruoyi.system.mapper.AsiccSuppliesMapper;
import com.ruoyi.system.model.wages.dao.AsiccDAO;
import com.ruoyi.system.service.AsiccSuppliesService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.Range;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author haiwe
 * @date 2024/7/5
 */
@Service
@RequiredArgsConstructor
public class AsiccSuppliesServiceImpl implements AsiccSuppliesService {

    private final AsiccSuppliesMapper asiccSuppliesMapper;

    @Override
    public Map<String, Object> getData(AsiccDAO asiccDAO) {
        //获取矿id
        String mineId = asiccDAO.getMineId();
        List<ActSuppliesMaterialCode> materialCodeList = asiccSuppliesMapper.selectByMineId(mineId);
        // 使用Lambda表达式按codeMin的长度从长到短排序
        Comparator<ActSuppliesMaterialCode> comparator = Comparator.comparing(code -> code.getCodeMin().length());
        materialCodeList.sort(comparator.reversed());

        if (materialCodeList.isEmpty()) {
            return getLevelByCode(mineId, asiccDAO);
        } else {
            return getElseByCode(mineId, asiccDAO, materialCodeList);
        }
    }

    /**
     * 处理特殊表不存在的矿id情况
     *
     * @param mineId 矿id（）
     * @return 一级，二级分类
     */
    private Map<String, Object> getLevelByCode(String mineId, AsiccDAO asiccDAO) {
        Map<String, Object> voMap = new HashMap<>();
        //获取相关信息-一级分类、二级分类
        List<ActSupplies> suppliesList = asiccSuppliesMapper.selectLevelInfo();
        Map<Object, String> suppliesMap = new HashMap<>();
        //存放suppliesList的一级分类和二级分类以及对应的id
        suppliesList.forEach(supplies -> {
            String oneLevel = supplies.getOneLevel();
            String twoLevel = supplies.getTwoLevel();
            Long id = supplies.getId();
            String finalLevel = oneLevel + "_" + twoLevel;
            suppliesMap.put(id, finalLevel);
        });
        List<ActSuppliesDetails> detailsList = asiccSuppliesMapper.select();
        //存放detailsList中的code
        Map<String, Object> codeMap = new HashMap<>();
        detailsList.forEach(v -> {
            String actCode = v.getCode();
            Long sid = v.getSid();
            codeMap.put(actCode, sid);
        });
        //特殊处理最后没有匹配上的数据的id为10
        ActSupplies actSupplies = asiccSuppliesMapper.selectByNum();

        for (String code : asiccDAO.getCodes()) {
            Map<String, Object> map = new HashMap<>();
            //存放基本生产原煤的一级、二级分类
            Map<String, Object> codeList1 = new HashMap<>();
            //存放基本生产洗煤的一级、二级分类
            Map<String, Object> codeList2 = new HashMap<>();
            //存放其他费用来源的一级、二级分类
            Map<String, Object> codeList3 = new HashMap<>();
            //存放制造费用-原煤
            Map<String, Object> codeList4 = new HashMap<>();
            String kmCode1 = StringUtils.substring(code, 0, 6);
            String kmCode2 = StringUtils.substring(code, 0, 4);
            String kmCode3 = StringUtils.substring(code, 0, 2);
            //处理林西矿
            if (mineId.equals("011704")) {
                Range<String> codeList = Range.between("30", "36");
                if (codeList.contains(code) || codeList.contains(kmCode3)) {
                    codeList1.put("一级分类", "配件");
                    codeList1.put("二级分类", "配件");
                    codeList2.put("一级分类", "配件");
                    codeList2.put("二级分类", "配件");
                    codeList3.put("一级分类", "其他材料");
                    codeList3.put("二级分类", "其他");
                    map.put("基本生产原煤", codeList1);
                    map.put("基本生产洗煤", codeList2);
                    map.put("其他费用来源", codeList3);
                    voMap.put(code, map);
                } else if (code.equals("6199") || kmCode2.equals("6199")) {
                    codeList1.put("一级分类", "其他材料");
                    codeList1.put("二级分类", "其他");
                    codeList2.put("一级分类", "其他材料");
                    codeList2.put("二级分类", "其他");
                    codeList3.put("一级分类", "其他材料");
                    codeList3.put("二级分类", "其他");
                    map.put("基本生产原煤", codeList1);
                    map.put("基本生产洗煤", codeList2);
                    map.put("其他费用来源", codeList3);
                    voMap.put(code, map);
                    //剩余情况（林西矿）
                } else
                    getCode(voMap, suppliesMap, codeMap, actSupplies,
                            code, map, codeList1, codeList2,
                            codeList3, kmCode1, kmCode2, kmCode3);
            }
            //处理东环坨
            else if (mineId.equals("010102")) {
                if (codeMap.containsKey(kmCode1)) {
                    disposeDong(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3, codeList4, kmCode1);
                } else if (codeMap.containsKey(kmCode2)) {
                    disposeDong(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3, codeList4, kmCode2);
                } else if (codeMap.containsKey(kmCode3)) {
                    disposeDong(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3, codeList4, kmCode3);
                } else {
                    getOtherTen(actSupplies, codeList1, codeList2, codeList3);
                    codeList4.put("一级分类", "其他材料");
                    codeList4.put("二级分类", "其他");
                    map.put("基本生产原煤", codeList1);
                    map.put("基本生产洗煤", codeList2);
                    map.put("其他费用来源", codeList3);
                    map.put("制造费用原煤", codeList4);
                    voMap.put(code, map);
                }
            }
            //其余情况
            else {
                getCode(voMap, suppliesMap, codeMap,
                        actSupplies, code, map,
                        codeList1, codeList2, codeList3,
                        kmCode1, kmCode2, kmCode3);
            }
        }

        return voMap;
    }

    /**
     * 处理东环坨剩余编码的匹配情况
     */
    private void disposeDong(Map<String, Object> voMap,
                             Map<Object, String> suppliesMap,
                             Map<String, Object> codeMap,
                             String code, Map<String, Object> map,
                             Map<String, Object> codeList1,
                             Map<String, Object> codeList2,
                             Map<String, Object> codeList3,
                             Map<String, Object> codeList4,
                             String kmCode) {
        Object sid = codeMap.get(kmCode);
        String finalLevel = suppliesMap.get(sid);
        String[] part = finalLevel.split("_");
        String oneLevel = part[0];
        String twoLevel = part[1];
        codeList1.put("一级分类", oneLevel);
        codeList1.put("二级分类", twoLevel);
        if (oneLevel.equals("油脂及乳化液") || oneLevel.equals("配件")) {
            codeList2.put("一级分类", oneLevel);
            codeList2.put("二级分类", twoLevel);
        } else {
            codeList2.put("一级分类", "其他材料");
            codeList2.put("二级分类", "其他");
        }
        if (oneLevel.equals("木材") || oneLevel.equals("支护用品") || oneLevel.equals("大型材料") || oneLevel.equals("劳保用品")) {
            codeList4.put("一级分类", oneLevel);
            codeList4.put("二级分类", twoLevel);
        } else {
            codeList4.put("一级分类", "其他材料");
            codeList4.put("二级分类", "其他");
        }
        codeList3.put("一级分类", "其他材料");
        codeList3.put("二级分类", "其他");
        map.put("基本生产原煤", codeList1);
        map.put("基本生产洗煤", codeList2);
        map.put("其他费用来源", codeList3);
        map.put("制造费用原煤", codeList4);
        voMap.put(code, map);
    }

    /**
     * 处理剩余通用情况-处理特殊表不存在的矿id
     */
    private void getCode(Map<String, Object> voMap,
                         Map<Object, String> suppliesMap,
                         Map<String, Object> codeMap,
                         ActSupplies actSupplies, String code,
                         Map<String, Object> map,
                         Map<String, Object> codeList1,
                         Map<String, Object> codeList2,
                         Map<String, Object> codeList3,
                         String kmCode1, String kmCode2, String kmCode3) {
        if (codeMap.containsKey(kmCode1)) {
            disposeCode(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3, kmCode1);
        } else if (codeMap.containsKey(kmCode2)) {
            disposeCode(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3, kmCode2);
        } else if (codeMap.containsKey(kmCode3)) {
            disposeCode(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3, kmCode3);
        } else {
            getOtherTen(actSupplies, codeList1, codeList2, codeList3);
            map.put("基本生产原煤", codeList1);
            map.put("基本生产洗煤", codeList2);
            map.put("其他费用来源", codeList3);
            voMap.put(code, map);
        }
    }

    /**
     * 处理编码-处理特殊表不存在的矿id
     */
    private void disposeCode(Map<String, Object> voMap,
                             Map<Object, String> suppliesMap,
                             Map<String, Object> codeMap, String code,
                             Map<String, Object> map,
                             Map<String, Object> codeList1,
                             Map<String, Object> codeList2,
                             Map<String, Object> codeList3,
                             String kmCode) {
        Object sid = codeMap.get(kmCode);
        String finalLevel = suppliesMap.get(sid);
        String[] part = finalLevel.split("_");
        String oneLevel = part[0];
        String twoLevel = part[1];
        codeList1.put("一级分类", oneLevel);
        codeList1.put("二级分类", twoLevel);
        if (oneLevel.equals("油脂及乳化液") || oneLevel.equals("配件")) {
            codeList2.put("一级分类", oneLevel);
            codeList2.put("二级分类", twoLevel);
        } else {
            codeList2.put("一级分类", "其他材料");
            codeList2.put("二级分类", "其他");
        }

        codeList3.put("一级分类", "其他材料");
        codeList3.put("二级分类", "其他");
        map.put("基本生产原煤", codeList1);
        map.put("基本生产洗煤", codeList2);
        map.put("其他费用来源", codeList3);

        voMap.put(code, map);
    }

    /**
     * 处理特殊表不存在的矿id，然后最后处理完编码为10的情况
     */
    private void getOtherTen(ActSupplies actSupplies,
                             Map<String, Object> codeList1,
                             Map<String, Object> codeList2,
                             Map<String, Object> codeList3) {
        String oneLevel = actSupplies.getSuppliesName();
        codeList1.put("一级分类", oneLevel);
        codeList1.put("二级分类", null);
        if (oneLevel.equals("油脂") || oneLevel.equals("配件")) {
            codeList2.put("一级分类", oneLevel);
            codeList2.put("二级分类", null);
        } else {
            codeList2.put("一级分类", "其他材料");
            codeList2.put("二级分类", "其他");
        }
        codeList3.put("一级分类", "其他材料");
        codeList3.put("二级分类", "其他");
    }

    /**
     * 处理特殊表存在的矿id情况
     *
     * @param mineId 矿id
     * @return 一级，二级分类
     */
    private Map<String, Object> getElseByCode(String mineId, AsiccDAO asiccDAO,
                                              List<ActSuppliesMaterialCode> materialCodeList) {
        Map<String, Object> voMap = new HashMap<>();
        Map<Object, String> suppliesMap = new HashMap<>();
        List<ActSupplies> suppliesList = asiccSuppliesMapper.select1();
        suppliesList.forEach(supplies -> {
            String oneLevel = supplies.getOneLevel();
            String oneName = supplies.getSuppliesName();
            suppliesMap.put(oneLevel, oneName);
        });
        for (String code : asiccDAO.getCodes()) {
            Map<String, Object> map = new HashMap<>();
            //存放基本生产原煤的一级、二级分类
            Map<String, Object> codeList1 = new HashMap<>();
            //存放基本生产洗煤的一级、二级分类
            Map<String, Object> codeList2 = new HashMap<>();
            //存放其他费用来源的一级、二级分类
            Map<String, Object> codeList3 = new HashMap<>();
            for (ActSuppliesMaterialCode materialCode : materialCodeList) {
                String kmCode1 = StringUtils.substring(code, 0, 6);
                String kmCode2 = StringUtils.substring(code, 0, 4);
                String kmCode3 = StringUtils.substring(code, 0, 2);
                String codeMin = materialCode.getCodeMin();
                String codeMax = materialCode.getCodeMax();
                Range<String> range = Range.between(codeMin, codeMax);
                String[] number1 = {"6101", "6102"};
                String[] number2 = {"30", "31", "32", "34", "35"};
                //处理吕家坨
                if (mineId.equals("011702")) {
                    //特殊情况
                    if (Arrays.asList(number1).contains(code) || Arrays.asList(number1).contains(kmCode2)) {
                        String twoLevel = materialCode.getMaterialName();
                        String codeType = materialCode.getMaterialCodeType();
                        String oneLevel = suppliesMap.get(codeType);
                        codeList1.put("一级分类", oneLevel);
                        codeList1.put("二级分类", twoLevel);
                        codeList2.put("一级分类", "油脂及乳化液");
                        codeList2.put("二级分类", "油脂");
                        codeList3.put("一级分类", "其他材料");
                        codeList3.put("二级分类", "其他");
                        map.put("基本生产原煤", codeList1);
                        map.put("基本生产洗煤", codeList2);
                        map.put("其他费用来源", codeList3);
                        voMap.put(code, map);
                    } else if (Arrays.asList(number2).contains(code) || Arrays.asList(number2).contains(kmCode3)) {
                        String twoLevel = materialCode.getMaterialName();
                        String codeType = materialCode.getMaterialCodeType();
                        String oneLevel = suppliesMap.get(codeType);
                        codeList1.put("一级分类", oneLevel);
                        codeList1.put("二级分类", twoLevel);
                        codeList2.put("一级分类", "配件");
                        codeList2.put("二级分类", "配件");
                        codeList3.put("一级分类", "其他材料");
                        codeList3.put("二级分类", "其他");
                        map.put("基本生产原煤", codeList1);
                        map.put("基本生产洗煤", codeList2);
                        map.put("其他费用来源", codeList3);
                        voMap.put(code, map);
                    } else {
                        getInfo(voMap, suppliesMap, code, map,
                                codeList1, codeList2, codeList3,
                                materialCode, kmCode1, kmCode2, kmCode3, range);
                    }
                }
                //其他情况
                else {
                    getInfo(voMap, suppliesMap, code, map,
                            codeList1, codeList2, codeList3,
                            materialCode, kmCode1, kmCode2, kmCode3, range);
                }
            }
        }
        return voMap;
    }

    /**
     * 处理特殊表中存在的矿-一级、二级分类
     */
    private void getInfo(Map<String, Object> voMap,
                         Map<Object, String> suppliesMap,
                         String code, Map<String, Object> map,
                         Map<String, Object> codeList1,
                         Map<String, Object> codeList2,
                         Map<String, Object> codeList3,
                         ActSuppliesMaterialCode materialCode,
                         String kmCode1, String kmCode2,
                         String kmCode3, Range<String> range) {
        if (range.contains(code)) {
            getLevelInfo(voMap, suppliesMap, code, map, codeList1, codeList2, codeList3, materialCode);
        } else if (range.contains(kmCode1)) {
            getLevelInfo(voMap, suppliesMap, code, map, codeList1, codeList2, codeList3, materialCode);
        } else if (range.contains(kmCode2)) {
            getLevelInfo(voMap, suppliesMap, code, map, codeList1, codeList2, codeList3, materialCode);
        } else if (range.contains(kmCode3)) {
            getLevelInfo(voMap, suppliesMap, code, map, codeList1, codeList2, codeList3, materialCode);
        } else {
            codeList1.put("一级分类", "其他材料");
            codeList1.put("二级分类", "其他");
            codeList2.put("一级分类", "其他材料");
            codeList2.put("二级分类", "其他");
            codeList3.put("一级分类", "其他材料");
            codeList3.put("二级分类", "其他");
            map.put("基本生产原煤", codeList1);
            map.put("基本生产洗煤", codeList2);
            map.put("其他费用来源", codeList3);
            voMap.put(code, map);
        }
    }

    /**
     * 获得一、二级分类的名称-特殊表存在矿id的情况
     */
    private void getLevelInfo(Map<String, Object> voMap,
                              Map<Object, String> suppliesMap,
                              String code, Map<String, Object> map,
                              Map<String, Object> codeList1,
                              Map<String, Object> codeList2,
                              Map<String, Object> codeList3,
                              ActSuppliesMaterialCode materialCode) {
        String twoLevel = materialCode.getMaterialName();
        String codeType = materialCode.getMaterialCodeType();
        String oneLevel = suppliesMap.get(codeType);
        codeList1.put("一级分类", oneLevel);
        codeList1.put("二级分类", twoLevel);
        if (oneLevel.equals("油脂及乳化液") || oneLevel.equals("配件")) {
            codeList2.put("一级分类", oneLevel);
            codeList2.put("二级分类", twoLevel);
        } else {
            codeList2.put("一级分类", "其他材料");
            codeList2.put("二级分类", "其他");
        }
        codeList3.put("一级分类", "其他材料");
        codeList3.put("二级分类", "其他");
        map.put("基本生产原煤", codeList1);
        map.put("基本生产洗煤", codeList2);
        map.put("其他费用来源", codeList3);
        voMap.put(code, map);
    }
}
