package com.ruoyi.system.service.impl;

import com.ruoyi.system.model.power.dto.DanHouInquireAboutSubjectAssistance;
import com.ruoyi.system.model.power.dto.DanHouInquireAboutSubjectAssistanceDto;
import com.ruoyi.system.model.power.dto.LinXiInquireAboutSubjectAssistance;
import com.ruoyi.system.model.power.model.DanHouVoucherModel;
import com.ruoyi.system.service.PowerVoucherConversionService;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 2024/8/1
 * 凭证转换---电力
 */
@Service
public class PowerVoucherConversionServiceImpl implements PowerVoucherConversionService {
    @Override
    /**
     * 林西凭证辅助核算调取部分
     */
    public List<LinXiInquireAboutSubjectAssistance> inquireAboutSubjectAssistance(List<LinXiInquireAboutSubjectAssistance> linXiInquireAboutSubjectAssistance) {
        Optional<String> first = linXiInquireAboutSubjectAssistance.parallelStream()
                .filter(v -> "6602002006024094".equals(v.getCode()))
                .map(v -> v.getMoney())
                .findFirst();
        for (int i = 0; i < linXiInquireAboutSubjectAssistance.size(); i++) {
            LinXiInquireAboutSubjectAssistance assistance = linXiInquireAboutSubjectAssistance.get(i);
            if (first.isPresent() && "6602002006024094".equals(assistance.getCode())) {
                linXiInquireAboutSubjectAssistance.remove(assistance);
                i++;
            }
            if (first.isPresent() && "6602002003".equals(assistance.getCode())) {
                assistance.setMoney(new BigDecimal(assistance.getMoney()).add(new BigDecimal(first.get())) + "");
            }
        }
        Map<String, List<String>> map = new HashMap<>();
        map.put("5002003", Arrays.asList("产品成本来源类别:0102_CW0025_100SX", "物料基本分类:MT01"));
        map.put("5002003_1", Arrays.asList("产品成本来源类别:0102_CW0025_100SX", "物料基本分类:MT0299"));
        map.put("5101003", Arrays.asList("产品成本来源类别:0102_CW0025_100SX", "物料基本分类:MT01"));
        map.put("6402003", Collections.singletonList("其他业务支出费用要素:03_CW0029_10070"));
        map.put("2211003002017", Arrays.asList("薪酬类别:01_CW0017_1003P", "薪酬列支渠道:030101_CW0018_1003Z"));
        map.put("2221001001001", Arrays.asList("核算岗位:01_CW0038_100BA", "票类:JX01_CW0049_100DR", "税目税率:VATR1"));
        Map<String, String> codeMap = new HashMap<>();
        codeMap.put("5002003", "5002003,基本生产_电力");
        codeMap.put("5002003_1", "5002003,基本生产_电力");
        codeMap.put("5101003", "5101003,制造费用_电费");
        codeMap.put("6601003", "6601003,销售费用_电费");
        codeMap.put("6602002003", "6602002003,管理费用_管理费_电费");
        codeMap.put("6402003", "6402003,其他业务成本_供电");
        codeMap.put("2211003002017", "2211003002017,应付职工薪酬_职工福利费_福利费支出_自办食堂支出");
        codeMap.put("2221001001001", "2221001001001,应交税费_应交增值税_进项税额_煤炭产品用");
        codeMap.put("2241005", "2241005,其他应付款_待分析");
        return linXiInquireAboutSubjectAssistance.parallelStream()
                .peek(v -> {
                    v.setAidedList(map.get(v.getCode()));
                    v.setCode(codeMap.get(v.getCode()));
                })
                .collect(Collectors.toList());
    }

    /**
     * 单侯凭证辅助核算调取部分
     *
     * @param danHouInquireAboutSubjectAssistanceDto
     * @return
     */
    @Override
    public List<DanHouInquireAboutSubjectAssistance> shanHouVoucherConversion(DanHouInquireAboutSubjectAssistanceDto danHouInquireAboutSubjectAssistanceDto) {
        List<DanHouInquireAboutSubjectAssistance> danHouInquireAboutSubjectAssistances = danHouInquireAboutSubjectAssistanceDto.getList();
        String date = danHouInquireAboutSubjectAssistanceDto.getYears();
        Map<String, DanHouVoucherModel> map = new HashMap<>();
        map.put("内部单位往来-北阳庄矿", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费北阳庄矿", "1135001002", Arrays.asList("客商:01120301")));
        map.put("内部单位往来-崔家寨矿", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费崔家寨矿", "1135001002", Arrays.asList("客商:011203")));
        map.put("内部单位往来-西细庄矿", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费西细庄矿", "1135001002", Arrays.asList("客商:011203")));
        map.put("内部单位往来-兴源矿", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费兴源矿", "1135001002", Arrays.asList("客商:011203")));
        map.put("内部单位往来-综合服务中心", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费综合服务中心", "1135001002", Arrays.asList("客商:011203")));
        map.put("基本生产-原煤", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费", "5002003", Arrays.asList("产品成本来源类别:0102_CW0025_100SX", "物料基本分类:MT01")));
        map.put("基本生产-选煤-自产", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费", "5002003", Arrays.asList("产品成本来源类别:04_CW0025_1005Q", "物料基本分类:MT0303")));
        map.put("制造费用-原煤", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费", "5101003", Arrays.asList("产品成本来源类别:0102_CW0025_100SX", "物料基本分类:MT01")));
        map.put("管理费用", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费", "6602002003", Arrays.asList("部门:01_011204")));
        map.put("其他业务成本-供电", new DanHouVoucherModel("转" + date + "电费结转外转电费", "6402003", Arrays.asList("其他业务支出费用要素:03_CW0029_10070")));
        map.put("其他业务成本-生活用电", new DanHouVoucherModel("转" + date + "生活用电电费", "6402003", Arrays.asList("其他业务支出费用要素:03_CW0029_10070")));
        map.put("安全费用-排水电费", new DanHouVoucherModel("转" + date + "排水电费", "4302002003004", Arrays.asList("投资项目费用要素:03_CW0010_100ZY", "安全生产费工程:03_CW0011_1000U")));
        map.put("研发支出-村庄群下压煤安全开采新工艺设计研究", new DanHouVoucherModel("转" + date + "研发支出电费", "5301002010", Arrays.asList("项目:020181_011204")));
        map.put("合计", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费", "2241005", null));
        map.put("销售费用", new DanHouVoucherModel("转" + date + "电费结转外转电费收内部单位电费", "6601003", null));
        List<DanHouInquireAboutSubjectAssistance> collect = danHouInquireAboutSubjectAssistances.parallelStream().filter(v -> !v.getLevelOneClassification().endsWith("-")).collect(Collectors.toList());
        for (DanHouInquireAboutSubjectAssistance assistance : collect) {
            String abs = assistance.getLevelOneClassification();
            if (assistance.getLevelTwoClassification() != null && !"".equals(assistance.getLevelTwoClassification())) abs = abs + "-" + assistance.getLevelTwoClassification();
            assistance.setDanHouVoucherModel(map.get(abs));
        }
        return collect;
    }
}
