package com.ruoyi.system.service.impl;

import com.alibaba.excel.EasyExcel;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.system.constant.CoalConstant;
import com.ruoyi.system.listener.TsExcelListener;
import com.ruoyi.system.mapper.SalaryMapper;
import com.ruoyi.system.model.salary.SalaryExcelModel;
import com.ruoyi.system.service.TsSalarySaveService;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;


@Service
public class TsSalarySaveServiceImpl implements TsSalarySaveService {

    public final static Set<String> COST_CHANNEL_SET = Stream.of("基本生产原煤", "基本生产洗煤", "制造费用原煤", "制造费用洗煤", "选混煤", "其他业务成本", "其他应付款", "管理费用", "销售费用", "其他业务支出").collect(Collectors.toSet());

    private final SalaryMapper salaryMapper;

    public TsSalarySaveServiceImpl(SalaryMapper salaryMapper) {
        this.salaryMapper = salaryMapper;
    }

    @Override
    @SneakyThrows
    public AjaxResult save(MultipartFile multipartFile, String salaryDate) {
        InputStream inputStream = multipartFile.getInputStream();
        TsExcelListener<SalaryExcelModel> tsExcelListener = new TsExcelListener<>();
        EasyExcel.read(inputStream, SalaryExcelModel.class, tsExcelListener).sheet(0).headRowNumber(3).doRead();
        List<SalaryExcelModel> salaryExcelModelList = tsExcelListener.getSalaryExcelModelList().stream().map(project -> {
            if (StringUtils.isNotBlank(project.getOrg())) {
                project.setOrg(project.getOrg().replaceAll("[.、0-9（）()]", StringUtils.EMPTY));
            }
            project.setBelongToCoal(CoalConstant.TS_MINE);
            project.setSalaryDate(salaryDate);
//            project.setCreator(SecurityUtils.getLoginUser().getUsername());
//            project.setModify(SecurityUtils.getLoginUser().getUsername());
            return project;
        }).collect(Collectors.toList());
        List<SalaryExcelModel> list = salaryExcelModelList.stream().filter(v -> !COST_CHANNEL_SET.contains(v.getOrg())).filter(v -> StringUtils.isNotBlank(v.getOrg())).filter(v -> !StringUtils.equalsAny(v.getOrg(), "唐山矿合计", "汇总合计")).collect(Collectors.toList());
        //如果重复导入先删除后增加
        salaryMapper.delete(salaryDate, CoalConstant.TS_MINE);
        salaryMapper.save(list);
        return AjaxResult.success("保存成功!");
    }

}
