package com.ruoyi.system.service.impl;

import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.mapper.WagesConversionMapper;
import com.ruoyi.system.model.wages.dao.WagesConversionData;
import com.ruoyi.system.model.wages.dto.WagesParamsDto;
import com.ruoyi.system.model.wages.vo.WagesConversionDataVO;
import com.ruoyi.system.service.WagesConversionService;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 2024/5/11
 */
@Service
public class WagesConversionServiceImpl implements WagesConversionService {
    private WagesConversionMapper wagesConversionMapper;

    private WagesConversionServiceImpl(WagesConversionMapper wagesConversionMapper) {
        this.wagesConversionMapper = wagesConversionMapper;
    }
    private static Map<String, String> roleIdMap = new HashMap<>();
    static {
        roleIdMap.put("10", "010101");
        roleIdMap.put("11", "011704");
        roleIdMap.put("12", "011204");
        roleIdMap.put("13", "011702");
        roleIdMap.put("14", "011701");
        roleIdMap.put("15", "011502");
        roleIdMap.put("16", "010102");
    }

    @Override
    public void save(WagesConversionDataVO wagesConversionDataVO) {
        wagesConversionMapper.deleteFromConversionToDate(wagesConversionDataVO.getYearMonth());
        wagesConversionMapper.save(wagesConversionDataVO.getJsonData(), wagesConversionDataVO.getYearMonth(), wagesConversionDataVO.getValue(), wagesConversionDataVO.getMineId());
    }

    @Override
    public List<WagesConversionData> selectWagesConversionDataList(WagesParamsDto wagesParamsDto) {
        String pageNum = wagesParamsDto.getPageNum();
        String pageSize = wagesParamsDto.getPageSize();
        String wageMonth = wagesParamsDto.getWageMonth();
        int n = (Integer.parseInt(pageNum) - 1) * Integer.parseInt(pageSize);
        SysUser user = SecurityUtils.getLoginUser().getUser();
        if (user.getRoles().size() != 0){
            SysRole sysRole = user.getRoles().get(0);
            String roleSort = sysRole.getRoleSort();
            wagesParamsDto.setMineId(roleIdMap.getOrDefault(roleSort, "0"));
        }
        return wagesConversionMapper.selectWagesConversionDataList(n, Integer.parseInt(pageSize), wageMonth, wagesParamsDto.getMineId());
    }

    @Override
    public void update(WagesConversionData wagesConversionData) {
        String jsonData = wagesConversionData.getJsonData();
        int id = wagesConversionData.getId();
        wagesConversionMapper.update(id, jsonData);
    }

    @Override
    public WagesConversionData selectWagesConversionDataById(int id) {
        return wagesConversionMapper.selectWagesConversionDataById(id);
    }

    @Override
    public AjaxResult recentFormulas(String mineId) {
        List<WagesConversionDataVO> list = wagesConversionMapper.selectWagesValueAndDate(mineId);
        list.removeAll(Collections.singleton(null));
        boolean b1 = list.parallelStream().anyMatch(v -> v.getYearMonth() == null || v.getYearMonth().indexOf("-") <= 0);
        if (b1) return new AjaxResult(500, "数据库中数据存在日期错误", null);
        Optional<Integer> max = list.parallelStream().map(v -> Integer.parseInt(v.getYearMonth().split("-")[0])).max((a, b) -> a - b);
        List<WagesConversionDataVO> collect;
        if (max.isPresent()) {
            collect = list.parallelStream().filter(v -> v.getYearMonth().split("-")[0].equals(max.get() + "")).collect(Collectors.toList());
        } else {
            return new AjaxResult(500, "数据库中没有数据", null);
        }
        if (collect.size() != 0) {
            collect = collect.parallelStream().sorted((a, b) -> Integer.parseInt(b.getYearMonth().split("-")[1]) - Integer.parseInt(a.getYearMonth().split("-")[1])).collect(Collectors.toList());
            return new AjaxResult(200, "成功", collect.get(0).getValue());
        }
        return new AjaxResult(500, "数据库中数据错误", null);
    }

    @Override
    public void deleteWagesDataById(int id) {
        wagesConversionMapper.deleteWagesDataById(id);
    }

}
